PROJECT=nrf24_basic
SOURCES=$(PROJECT).cc Arduino/hardware/arduino/avr/cores/arduino/wiring.c Arduino/hardware/arduino/avr/cores/arduino/hooks.c Arduino/hardware/arduino/avr/cores/arduino/wiring_digital.c RF24/RF24.cpp
MMCU=attiny44
SMCU=t44
F_CPU = 8000000
CC=avr-c++
ifeq (1,${RADIO_NUM})
    RADIO_NUM=1
else
    RADIO_NUM=0
endif

SPI_MOSI=9
SPI_MISO=0
SPI_SCK=10

DEFINE=-DF_CPU=$(F_CPU) -DRADIO_NUM=$(RADIO_NUM) -DARDUINO=150 -DSOFTSPI=1 -DSOFT_SPI_MISO_PIN=$(SPI_MISO) -DSOFT_SPI_MOSI_PIN=$(SPI_MOSI) -DSOFT_SPI_SCK_PIN=$(SPI_SCK) -D_SPI=spi -DSPI_MODE=0 -DMINIMAL -DSOFTSPI_NO_INLINE
CFLAGS=-mmcu=$(MMCU) $(DEFINE) -Wall -Os -std=c++11 -Wno-write-strings -fdata-sections -ffunction-sections -Wl,-gc-sections
LFLAGS=

$(PROJECT).$(RADIO_NUM).hex: $(PROJECT).$(RADIO_NUM).out
	avr-objcopy -O ihex $(PROJECT).$(RADIO_NUM).out $(PROJECT).cc.$(RADIO_NUM).hex;\
	avr-size --mcu=$(MMCU) --format=avr $(PROJECT).$(RADIO_NUM).out
 
$(PROJECT).$(RADIO_NUM).out: $(SOURCES)
	$(CC) $(CFLAGS) -I./ -I./DigitalIO/src/ -I./Arduino/hardware/arduino/avr/cores/arduino/ -I./Arduino/hardware/arduino/avr/variants/tiny14/ -I./RF24/ -o $(PROJECT).$(RADIO_NUM).out $(SOURCES) $(LFLAGS)

program-bsd: $(PROJECT).$(RADIO_NUM).hex
	avrdude -p $(SMCU) -c bsd -U flash:w:$(PROJECT).cc.$(RADIO_NUM).hex

program-dasa: $(PROJECT).$(RADIO_NUM).hex
	avrdude -p $(SMCU) -P /dev/ttyUSB0 -c dasa -U flash:w:$(PROJECT).cc.$(RADIO_NUM).hex

program-avrisp2: $(PROJECT).$(RADIO_NUM).hex
	avrdude -p $(SMCU) -P usb -c avrisp2 -U flash:w:$(PROJECT).cc.$(RADIO_NUM).hex

program-usbtiny: $(PROJECT).$(RADIO_NUM).hex
	avrdude -p $(SMCU) -P usb -c usbtiny -U flash:w:$(PROJECT).cc.$(RADIO_NUM).hex

program-dragon: $(PROJECT).$(RADIO_NUM).hex
	avrdude -p $(SMCU) -P usb -c dragon_isp -U flash:w:$(PROJECT).cc.$(RADIO_NUM).hex
