// from Neil's code
#pragma once

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
//
// macros
//
#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay in us for 9600 with overhead at 8 MHz
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

//
// define serial
//
#define serial_port PORTA
#define serial_direction DDRA
#define serial_pins PINA
#define serial_pin_in (1 << PA2)
#define serial_pin_out (1 << PA3)

#ifdef __cplusplus
// extern "C"{
#endif

void serial_flush(){
  char_delay();
}

//
// get_char
//    read character into rxbyte on pins pin
//    assumes line driver (inverts bits)
//
//
void serial_read(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
  *rxbyte = 0;
  while (pin_test(*pins,pin))
     //
     // wait for start bit
     //
     ;
  //
  // delay to middle of first data bit
  //
  half_bit_delay();
  bit_delay();
  //
  // unrolled loop to read data bits
  //
#ifndef SERIAL_NO_UNROLL
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 0);
  else
     *rxbyte |= (0 << 0);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 1);
  else
     *rxbyte |= (0 << 1);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 2);
  else
     *rxbyte |= (0 << 2);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 3);
  else
     *rxbyte |= (0 << 3);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 4);
  else
     *rxbyte |= (0 << 4);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 5);
  else
     *rxbyte |= (0 << 5);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 6);
  else
     *rxbyte |= (0 << 6);
  bit_delay();
  if pin_test(*pins,pin)
     *rxbyte |= (1 << 7);
  else
     *rxbyte |= (0 << 7);
  bit_delay();
#else
  for(unsigned char i = 0; i < 8; ++i){
    if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
    else
      *rxbyte |= (0 << 0);
    bit_delay();
  }
#endif
  //
  // wait for stop bit
  //
  // bit_delay();
  half_bit_delay();
}

//
// put_char
//    send character in txchar on port pin
//    assumes line driver (inverts bits)
//
void serial_put(volatile unsigned char *port, unsigned char pin, char txchar) {
  //
  // start bit
  //
  clear(*port,pin);
  bit_delay();
  //
  // unrolled loop to write data bits
  //
#ifndef SERIAL_NO_UNROLL
  if bit_test(txchar,0)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,1)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,2)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,3)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,4)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,5)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,6)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
  if bit_test(txchar,7)
     set(*port,pin);
  else
     clear(*port,pin);
  bit_delay();
#else
  for(unsigned char i = 0; i < 8; ++i){
    if bit_test(txchar,0)
      set(*port,pin);
    else
      clear(*port,pin);
    bit_delay();
  }
#endif
  //
  // stop bit
  //
  set(*port,pin);
  bit_delay();
  //
  // char delay
  //
  bit_delay();
}

void serial_write(volatile unsigned char *port, unsigned char pin, char *str) {
  //
  // print a null-terminated string
  //
  static unsigned char index;
  index = 0;
  do {
    serial_put(port, pin, str[index]);
    ++index;
  } while (str[index] != 0);
}

void serial_begin(){
  set(serial_port, serial_pin_out);
  output(serial_direction, serial_pin_out);
}

void serial_read(char *chr){
  serial_read(&serial_pins, serial_pin_in, chr);
}
char serial_read() {
  char chr;
  serial_read(&chr);
  return chr;
}
void serial_put(char chr){
  serial_put(&serial_port, serial_pin_out, chr);
}
void serial_putc(unsigned char chr){
  serial_put(&serial_port, serial_pin_out, chr);
}
void serial_digit(unsigned char d){
  static const unsigned char digits[] = "0123456789ABCDEF";
  serial_putc(digits[d]);
}
template <class D>
void serial_putd(D num){
  // sign
  if(num < 0){
    serial_putc('-');
    num = -num;
  }
  // count and store digits
  unsigned char numDigits = 0;
  static char digits[sizeof(D)]; // could be lower in size since it's digits, not bits!
  do {
    digits[numDigits] = num % 10;
    ++numDigits;
    num /= 10;
  } while(num > 0);
  // write digits
  for(unsigned char i = 0; i < numDigits; ++i){
    serial_digit(digits[numDigits - i - 1]);
  }
}
void serial_write(char *str){
  serial_write(&serial_port, serial_pin_out, str);
}
// inline void serial_writeln(char *str) __attribute__((always_inline));
inline void serial_writeln(char *str) {
  serial_write(str);
  serial_write("\r\n");
}

#ifdef __cplusplus
//}
#endif
